/************************************************************
 * @file: FBK.c
 * @author: Young Leon
 * @version: V1.0
 * @data: 2022/10/18
 * @brief: CAPCOM module function for BLDC control. Including
 *          perceptual rotor position and speed and commutating
 *          by using MOT.c
 * @note:
 * @Copyright (C) 2022 Novosense All rights reserved.
 ************************************************************/

/************************************************************
 ************************************************************
 * @par Edition History
 * -V1.0  2022.10.18
 *        -first version for NSUC16x0 sensor based BLDC Motor with
 *          six step commutation.
 *
 ************************************************************/

#include "FBK.h"
/*    Definition    */
#define CAPCOM_OVF0_INT (0x1 << 0)
#define CAPCOM_CMP0_INT (0x1 << 1)
#define CAPCOM_CAP0_INT (0x1 << 2)
#define CAPCOM_CVF0_INT (0x1 << 3)
#define CAPCOM_OVF1_INT (0x1 << 4)
#define CAPCOM_CMP1_INT (0x1 << 5)
#define CAPCOM_CAP1_INT (0x1 << 6)
#define CAPCOM_CVF1_INT (0x1 << 7)
#define CAPCOM_OVF2_INT (0x1 << 8)
#define CAPCOM_CMP2_INT (0x1 << 9)
#define CAPCOM_CAP2_INT (0x1 << 10)
#define CAPCOM_CVF2_INT (0x1 << 11)
#define CAPCOM_RP_INT   (0x1 << 12)
#define CAPCOM_WP_INT   (0x1 << 13)

/*    Global Variables    */

/**
 *  @brief Rotor Position Value, indicate hall sensor value (HC, HB, HA)
          | 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 |
          |---|---|---|---|---|---|---|---|
          | 0 | 0 | 0 | 0 | 0 | HC| HB| HA|
           Valid Range: 0-7
*/
uint8_t g_FBKRotorPosition;

/**
 *  @brief Rotor speed value
 */
uint16_t g_FBKRotorSpeed;

#if FBK_CAPCOM_MODE == 0
uint16_t g_FBKRotorSpeedBuff;
#elif FBK_CAPCOM_MODE == 1
uint16_t g_FBKRotorSpeedBuff[6];
#endif

uint16_t g_FBKStartupTable[STARTUP_TABLE_LEN] = {0};

uint16_t g_FBKStartupDutyTable[STARTUP_TABLE_LEN] = {0};

/**
 *  @brief counts CAPCOM overflow
 */
uint8_t g_FBKRotorSpeedOV;

/**
 * @brief the duration between zero crossing and last comparing
 *
 */
uint16_t g_FBKZeroCrossTime = 0;

/**
 * @brief count startup steps
 *
 */
uint16_t g_FBKStartupSteps = 0;

/**
 * @brief the counter of last step
 *
 */
uint16_t g_FBKLastStep = 0;

/**
 * @brief CMP0 last compare value
 *
 */
uint16_t g_FBKLastCompare = 0;

/**
 * @brief CMP0 next compare value
 *
 */
uint16_t g_FBKNextCompare = 0;

/************************************************************
 * @brief: Init CAPCOM module for Sensorless controlling.
 * @return <None>
 ************************************************************/
void FBK_Init(void)
{
    SYSCTRL->LKKEYR             = SYSCTRL_UNLOCK_KEY;
    SYSCTRL->APBCGR_b.CAPCMPCEN = 1;
    SYSCTRL->AHBCGR_b.DACCEN    = 1;

    /* CAPCOM config */
    CAPCOM->CCDR = FBK_CAPCOM_CLK_PS; // CAPCOM prescaler

    CAPCOM->CCR_b.CIS0 = 1; // CAPCOM source:0:GPIO 1:BEMFC
    CAPCOM->CCR_b.CIS1 = 1;
    CAPCOM->CCR_b.CIS2 = 1;
    CAPCOM->CCR_b.CIF0 = FBK_CAPCOM_FILTER_TIME; // CAPCOM input filter time 0.264us*N
    CAPCOM->CCR_b.CIF1 = FBK_CAPCOM_FILTER_TIME;
    CAPCOM->CCR_b.CIF2 = FBK_CAPCOM_FILTER_TIME;

    CAPCOM->CCR_b.ACM  = 0; // 0:standard 1:advance
    CAPCOM->CCR_b.IAM0 = 3; // CAPCOM capture edge
    CAPCOM->CCR_b.IAM1 = 3; // 0:disable 1:rising 2:falling 3:both
    CAPCOM->CCR_b.IAM2 = 3;

    //    CAPCOM->IEN_b.CAP0 = 1; // enable CAP0 interrupt
    //    CAPCOM->IEN_b.CAP1 = 1;
    //    CAPCOM->IEN_b.CAP2 = 1;
    //    CAPCOM->IEN_b.OVF0 = 1;
    //    CAPCOM->IEN_b.OVF1 = 1;
    //    CAPCOM->IEN_b.OVF2 = 1;

    LL_NVIC_EnableIRQ(CAPCOM0_IRQn);
    LL_NVIC_EnableIRQ(CAPCOM1_IRQn);
    LL_NVIC_EnableIRQ(CAPCOM2_IRQn);
    LL_NVIC_SetPriority(CAPCOM0_IRQn, 0, CAPCOM0_IRQ_PRIO);
    LL_NVIC_SetPriority(CAPCOM1_IRQn, 0, CAPCOM1_IRQ_PRIO);
    LL_NVIC_SetPriority(CAPCOM2_IRQn, 0, CAPCOM2_IRQ_PRIO);

    /* Init BEMFC module*/
    /* Virtual start conncet */
    BEMFC->CR2_b.BRM   = 0;
    BEMFC->CR2_b.BIS0  = 0;
    BEMFC->CR2_b.BIS1  = 0;
    BEMFC->CR2_b.BIS2  = 0;
    BEMFC->CR2_b.HYST0 = 3;
    BEMFC->CR2_b.HYST1 = 3;
    BEMFC->CR2_b.HYST2 = 3;

    /* BEMFC trigger source initial*/
    BEMFC->CR1_b.ITS0 = 0;
    BEMFC->CR1_b.ITS1 = 0;
    BEMFC->CR1_b.ITS2 = 0;
    BEMFC->CR1_b.DTP0 = 2; // BEMFC channel detection period: 0: disable
    // 1: low level 2: high level 4: low and high level
    BEMFC->CR1_b.DTP1 = 2;
    BEMFC->CR1_b.DTP2 = 2;
    //    BEMFC->CR1_b.DTS0 = 8;
    //    BEMFC->CR1_b.DTS1 = 10;
    //    BEMFC->CR1_b.DTS2 = 9;

    /*blanking time and deglitch time setting*/
    BEMFC->CR0_b.DGL0  = FBK_BEMFC_DEGLITCH_FILTER_TIME;
    BEMFC->CR0_b.DGL1  = FBK_BEMFC_DEGLITCH_FILTER_TIME;
    BEMFC->CR0_b.DGL2  = FBK_BEMFC_DEGLITCH_FILTER_TIME;
    BEMFC->CR0_b.BLKT0 = FBK_BEMFC_BLANKING_TIME;
    BEMFC->CR0_b.BLKT1 = FBK_BEMFC_BLANKING_TIME;
    BEMFC->CR0_b.BLKT2 = FBK_BEMFC_BLANKING_TIME;

    BEMFC->CR0_b.EN0 = 1;
    BEMFC->CR0_b.EN1 = 1;
    BEMFC->CR0_b.EN2 = 1;

    delay_ms(5); // wait 5ms for HSBVDD ramp-up, read a valid rotor position

}

/************************************************************
 * @brief: Fetch speed of sensorless.
 * @return <None>
 ************************************************************/
void FBK_ReadRotorSpeed(void)
{
    // @todo: consider buffer of last six step counter
    g_FBKRotorSpeed = __ssat(g_FBKLastStep * 6, 16);
}

/************************************************************
 * @brief: Enable BEMFC commutation
 * @return <None>
 ************************************************************/
void FBK_EnableBEMFCCommutation(void)
{
    g_FBKZeroCrossTime = 0;
    g_FBKStartupSteps  = 0;
    g_FBKLastStep      = 0;
    g_FBKLastCompare   = 0;
    g_FBKNextCompare   = MCM_ALIGN_TIME;

    CAPCOM->CCMPR0_b.CMPR = MCM_ALIGN_TIME;
    CAPCOM->IEN_b.CMP0    = 1;
    CAPCOM->CCR_b.EN      = 1;
}

/************************************************************
 * @brief: Disable BEMFC commutation
 * @return <None>
 ************************************************************/
void FBK_DisableBEMFCCommutation(void)
{
    CAPCOM->CCR_b.EN = 0;
    CAPCOM->IEN      = 0;
}

/************************************************************
 * @brief: calculate the time of next step.
 * @return <None>
 ************************************************************/
uint16_t FBK_GetNextStep(void)
{
    uint16_t nextStep = 0;

    if (g_FBKStartupSteps < STARTUP_TABLE_LEN) { // during startup
        nextStep      = g_FBKStartupTable[g_FBKStartupSteps];
        g_MCMPwmWidth = g_FBKStartupDutyTable[g_FBKStartupSteps];
        g_FBKStartupSteps++;
    } else { // in case of losing zero crossing
        nextStep = g_FBKLastStep + (g_FBKLastStep >> 2);
    }
    return nextStep;
}

/************************************************************
 * @brief: Set next commutating compare value
 * @param <uint16_t> captureVal
 * @return <None>
 ************************************************************/
void FBK_SetNextCommutation(uint16_t captureVal)
{
    g_FBKZeroCrossTime = captureVal - g_FBKLastCompare;

    if (g_FBKZeroCrossTime > (g_FBKLastStep >> 2)) {
        g_FBKNextCompare = g_FBKLastCompare + g_FBKLastStep / 2;
        g_FBKNextCompare += g_FBKZeroCrossTime;
    } else {
        g_FBKNextCompare = g_FBKLastCompare + g_FBKLastStep / 2;
        g_FBKNextCompare += g_FBKLastStep / 4;
    }
    CAPCOM->CCMPR0_b.CMPR = g_FBKNextCompare;
}

/************************************************************
 * @brief: CAPCOM0 interrupt function for standard mode.
 * @return <None>
 ************************************************************/
void CAPCOM0_IRQHandler(void)
{
    static uint16_t tmpCaptureValue = 0;

    if (CAPCOM->EIPND_b.CMP0) {
        MOT_CommutatePhases();
        CAPCOM->IEN &= 0xFFFFFBBB;
        CAPCOM->EIPND = 0x0446;
        /* calculate next step */
        g_FBKLastStep    = g_FBKNextCompare - g_FBKLastCompare;
        g_FBKLastCompare = g_FBKNextCompare;
        g_FBKNextCompare = g_FBKLastCompare + FBK_GetNextStep();

        CAPCOM->CCMPR0_b.CMPR = g_FBKNextCompare;
    }

    /* handle capture*/
    if (CAPCOM->EIPND_b.CAP0) {
        GPIO->PDO_b.DO5 = ~GPIO->PDO_b.DO5;
        tmpCaptureValue = (uint16_t)CAPCOM->CCAPR0_b.CAPR;
        FBK_SetNextCommutation(tmpCaptureValue);
        CAPCOM->IEN_b.CAP0 = 0;
        CAPCOM->EIPND      = 0x444;
    }
}

/************************************************************
 * @brief: CAPCOM1 interrupt function for standard mode.
 * @return <None>
 ************************************************************/
void CAPCOM1_IRQHandler(void)
{
    static uint16_t tmpCaptureValue = 0;
    static uint16_t lastCapture0    = 0;
    static uint16_t lastCapture1    = 0;
    static uint8_t toggle           = 0;

    /* handle capture*/
    if (CAPCOM->EIPND_b.CAP1) {
        GPIO->PDO_b.DO8 = ~GPIO->PDO_b.DO8;
        tmpCaptureValue = (uint16_t)CAPCOM->CCAPR1_b.CAPR;
        FBK_SetNextCommutation(tmpCaptureValue);
        CAPCOM->IEN_b.CAP1 = 0;
        CAPCOM->EIPND      = 0x444;
    }
}

/************************************************************
 * @brief: CAPCOM2 interrupt function for standard mode.
 * @return <None>
 ************************************************************/
void CAPCOM2_IRQHandler(void)
{
    static uint16_t tmpCaptureValue = 0;
    static uint16_t lastCapture0    = 0;
    static uint16_t lastCapture1    = 0;
    static uint8_t toggle           = 0;

    /* handle capture*/
    if (CAPCOM->EIPND_b.CAP2) {
        GPIO->PDO_b.DO9 = ~GPIO->PDO_b.DO9;
        tmpCaptureValue = (uint16_t)CAPCOM->CCAPR2_b.CAPR;
        FBK_SetNextCommutation(tmpCaptureValue);
        CAPCOM->IEN_b.CAP2 = 0;
        CAPCOM->EIPND      = 0x444;
    }
}


/************************************************************
 * @brief: Init the table of ramp up during startup
 * @return <None>
 ************************************************************/
void FBK_InitStartupTable(void)
{
    g_FBKStartupTable[0]     = MCM_SPEED_FACTOR / MCM_STARTUP_INITIAL_SPEED;
    g_FBKStartupDutyTable[0] = MCM_STARTUP_DUTY;
    for (int16_t i = 1; i < STARTUP_TABLE_LEN; i++) {
        g_FBKStartupTable[i]     = QMATH_UQMUL16(g_FBKStartupTable[i - 1], _UQ16(MCM_STARTUP_ACC_RATIO), UQ16);
        g_FBKStartupDutyTable[i] = QMATH_UQMUL16(g_FBKStartupDutyTable[i - 1], _UQ15(MCM_STARTUP_DUTY_ACC), UQ15);
    }
}