/************************************************************
 * @file: MCM.c
 * @author: Young Leon
 * @version: V1.0
 * @data: 2022/10/18
 * @brief: Motor Control Meachine file. Implement control state
 *          meachine and speed controller.
 * @note:
 * @Copyright (C) 2022 Novosense All rights reserved.
 ************************************************************/
/************************************************************
 ************************************************************
 * @par Edition History
 * -V1.0  2022.10.18
 *        -first version for NSUC16x0 sensor based BLDC Motor with
 *          six step commutation.
 *
 ************************************************************/

#include "MCM.h"

/**********************Public Variable********************************/
/**
 *  @brief EPWM width set to the motor
 */
uint16_t g_MCMPwmWidth = 0;

/**********************Private Variable********************************/
/**
 * @brief target speed in RPM for speed control
 *
 */
static uint16_t g_MCMTagetRpm = 0;

/**
 * @brief current motor speed in RPM
 *
 */
static uint16_t g_MCMCurRpm = 0;

/**
 * @brief global incremental pid variable for speed control
 *
 */
static IncPid_t g_MCMIncPid;

/**
 * @brief state variable for motor control meachine
 *
 */
static enum MCM_StateMeachine_t g_MCMState = MCM_OFF;

/**********************Public Function********************************/

/************************************************************
 * @brief: Set Motor Status to MCM_OFF
 * @return <None>
 ************************************************************/
void MCM_SetMotorOff(void) { g_MCMState = MCM_OFF; }

/************************************************************
 * @brief: Start Clockwise rotation
 * @return <None>
 ************************************************************/
void MCM_StartCWRotate(void) { g_MCMState = MCM_START_CW; }

/************************************************************
 * @brief: Start Counter-Clockwise rotation
 * @return <None>
 ************************************************************/
void MCM_StartCCWRotate(void) { g_MCMState = MCM_START_CCW; }

/************************************************************
 * @brief: Start Clockwise step rotation
 * @return <None>
 ************************************************************/
void MCM_StartCWStep(void) { g_MCMState = MCM_START_CW_STEP; }

/************************************************************
 * @brief: Start Counter-Clockwise step rotation
 * @return <None>
 ************************************************************/
void MCM_StartCCWStep(void) { g_MCMState = MCM_START_CCW_STEP; }

/************************************************************
 * @brief: Motor one clockwise step
 * @return <None>
 ************************************************************/
void MCM_GoOneCWStep(void) { g_MCMState = MCM_CW_STEP; }

/************************************************************
 * @brief: Motor one counter-clockwise tep
 * @return <None>
 ************************************************************/
void MCM_GoOneCCWStep(void) { g_MCMState = MCM_CCW_STEP; }

/************************************************************
 * @brief: Directly brake motor
 * @return <None>
 ************************************************************/
void MCM_DirectBrake(void) { g_MCMState = MCM_DIRECT_BRAKE; }

/************************************************************
 * @brief: Set motor speed down step by step
 * @return <None>
 ************************************************************/
void MCM_ControlBrake(void) { g_MCMState = MCM_CTRL_BRAKE; }

/************************************************************
 * @brief: Fetch status of state machine
 * @return <g_MCMState> current status of state machine
 ************************************************************/
uint8_t MCM_GetStatus(void) { return g_MCMState; }

/************************************************************
 * @brief: Set target speed in RPM
 * @param <uint16_t> tarRPM
 * @return <None>
 ************************************************************/
void MCM_SetTargetRPM(uint16_t tarRPM) { g_MCMTagetRpm = tarRPM; }

/************************************************************
 * @brief: Get current target speed in RPM
 * @return <g_MCMTagetRpm>
 ************************************************************/
uint16_t MCM_GetTargetRPM(void) { return g_MCMTagetRpm; }

/************************************************************
 * @brief: Get current speed in RPM
 * @return <g_MCMCurRpm>
 ************************************************************/
uint16_t MCM_GetCurrentRPM(void) { return g_MCMCurRpm; }

/**********************Private Function********************************/

/************************************************************
 * @brief: Speed controller using incremental PID. Output is
 *          EWPM duty.
 * @return <None>
 ************************************************************/
static void MCM_PiControlSpeed()
{
    static IQ15_t tmpOutput = 0;
    static uint16_t rampCnt = 0;
    /* calculate target speed ramp up/down speed*/
    // g_MCMIncPid.refTarget = (IQ15_t)(((int32_t)g_MCMTagetRpm * 32768) / MCM_MAX_RPM);

    FBK_ReadRotorSpeed();

    /* transfer g_FBKRotorSpeed into physic speed(RPM) */
    g_MCMCurRpm       = (MCM_SPEED_FACTOR / g_FBKRotorSpeed);
    g_MCMIncPid.input = (IQ15_t)(((int32_t)g_MCMCurRpm * 32768) / MCM_MAX_RPM);
    /* calculate error*/
    g_MCMIncPid.curErr = QMATH_IQSUB16_SAT(g_MCMIncPid.curTarget, g_MCMIncPid.input);
    /* calculate partial ouput */
    tmpOutput = QMATH_IQMUL16(g_MCMIncPid.kP, QMATH_IQSUB16_SAT(g_MCMIncPid.curErr, g_MCMIncPid.lastErr), IQ15) +
                QMATH_IQMUL16(g_MCMIncPid.kI, g_MCMIncPid.curErr, IQ15);
    /* assign current error to last error*/
    g_MCMIncPid.lastErr = g_MCMIncPid.curErr;
    /* incremental output */
    g_MCMIncPid.ouput = QMATH_IQADD16(g_MCMIncPid.ouput, tmpOutput);

    /* Limit output */
    if (g_MCMIncPid.ouput <= 32768 * 5 / 100) {
        g_MCMIncPid.ouput = 32768 * 5 / 100;
    } else if (g_MCMIncPid.ouput > 32768 * MOT_EPWM_MAX_OUTPUT / 100) {
        g_MCMIncPid.ouput = 32768 * MOT_EPWM_MAX_OUTPUT / 100;
    }
#if (MOT_FRWHL == 6)
    /* transfer Q15 format into duty*/
    g_MCMPwmWidth =
        MOT_EPWM_PERIOD_COMPARE - (uint16_t)((int32_t)((int32_t)g_MCMIncPid.ouput * MOT_EPWM_PERIOD_COMPARE) / 32768);

#else
    g_MCMPwmWidth = (uint16_t)((int32_t)((int32_t)g_MCMIncPid.ouput * MOT_EPWM_PERIOD_COMPARE) / 32768);
#endif
}

/************************************************************
 * @brief: Reset speed controller variable
 * @return <None>
 ************************************************************/
static void MCM_ResetSpeedControl()
{
    g_MCMIncPid.curErr    = 0;
    g_MCMIncPid.lastErr   = 0;
    g_MCMIncPid.preErr    = 0;
    g_MCMIncPid.kP        = _IQ15(MCM_GAINP);
    g_MCMIncPid.kI        = _IQ15(MCM_GAINI);
    g_MCMIncPid.ouput     = 0;
    g_MCMIncPid.curTarget = 0;
    g_MCMIncPid.refTarget = (IQ15_t)(((int32_t)g_MCMTagetRpm * 32768) / MCM_MAX_RPM);
    g_MCMIncPid.input     = 0;
    g_MCMIncPid.dTarget   = 0;
    g_MCMIncPid.rampOrder = MCM_RAMP_ORDER;
    g_MCMIncPid.targetAdd = 0;
    g_MCMPwmWidth         = 0;
    g_FBKRotorSpeedOV     = 0;
}

/************************************************************
 * @brief: motor control meachine. Allowable entering state only
 *         can be MCM_OFF,MCM_DIRECT_BRAKE, MCM_CTRL_BRAKE, MCM_START_CCW,
 *          MCM_START_CW, MCM_START_CCW_STEP, MCM_CCW_STEP, MCM_START_CW_STEP,
 *          MCM_CW_STEP.
 * @return <None>
 ************************************************************/
static void MCM_StateMeachine()
{
    static uint16_t alignCnt = 0;

    if (g_MOTFaultStatus != 0) { // detect fault
        g_MCMState       = MCM_OFF;
        g_MOTFaultStatus = 0; // Clear Fault Status @todo: use fault clearing function to clear fault flag
    }

    switch (g_MCMState) {
    case MCM_OFF:
        MOT_SwitchMotorOff();
        MCM_ResetSpeedControl();
        g_MCMState = MCM_OFF;
        break;
    case MCM_START_CCW:
#if MCM_SPEED_CONTROL
        MCM_ResetSpeedControl();
#else
        MOT_ChangeEpwmWidth();
#endif
        g_FBKRotorPosition = MOT_SECTOR_0;
        g_MCMPwmWidth      = MCM_STARTUP_DUTY;
        MOT_ChangeEpwmWidth();
        MOT_StartCCWRotation();
        g_MCMState = MCM_CCW;
        break;
    case MCM_START_CW:
#if MCM_SPEED_CONTROL
        MCM_ResetSpeedControl();
#else
        MOT_ChangeEpwmWidth();
#endif
        g_FBKRotorPosition = MOT_SECTOR_0;
        g_MCMPwmWidth      = MCM_STARTUP_DUTY;
        MOT_ChangeEpwmWidth();
        MOT_StartCWRotation();
        g_MCMState = MCM_CW;
        break;
    case MCM_CW:
    case MCM_CCW:
        MOT_ChangeEpwmWidth();

#if MCM_SPEED_CONTROL
        //@todo: Add Speed Control
//        MCM_PiControlSpeed();
//        MOT_ChangeEpwmWidth();
#else
        MOT_ChangeEpwmWidth();
#endif
        break;
    case MCM_START_CCW_STEP:
        MOT_StartCCWStep();
        MOT_ChangeEpwmWidth();
        g_MCMState = MCM_CW_CCW_STEP_IDLE;
        break;
    case MCM_CCW_STEP:
        MOT_GoOneStepCCW();
        g_MCMState = MCM_CW_CCW_STEP_IDLE;
        break;
    case MCM_START_CW_STEP:
        MOT_StartCWStep();
        MOT_ChangeEpwmWidth();
        g_MCMState = MCM_CW_CCW_STEP_IDLE;
        break;
    case MCM_CW_STEP:
        MOT_GoOneStepCW();
        g_MCMState = MCM_CW_CCW_STEP_IDLE;
        break;
    case MCM_CW_CCW_STEP_IDLE:
        break;
    case MCM_OFF_STEP:
        MOT_LeaveMotorStep();
        g_MCMState = MCM_OFF;
        break;
    case MCM_DIRECT_BRAKE:
        MOT_BrakeMotor();
        MCM_ResetSpeedControl();
        g_MCMState = MCM_OFF;
        break;
    case MCM_CTRL_BRAKE:
#if MCM_SPEED_CONTROL
        MOT_BrakeMotor();
        MCM_ResetSpeedControl();
        g_MCMState = MCM_OFF;
//        g_MCMIncPid.refTarget = 0;
//        MCM_PiControlSpeed();
//        MOT_ChangeEpwmWidth();
//        if (g_FBKRotorSpeedOV > 2) {
//            MOT_BrakeMotor();
//            g_MCMCurRpm = 0;
//            g_MCMState  = MCM_OFF;
//        }
#else
        MOT_BrakeMotor();
        g_MCMState = MCM_OFF;
#endif
        break;
    defualt:
        break;
    }
}

#define CLEAR_EPWM_ENDPERIOD_INT (0x01 << 0)
void EPWM0_IRQHandler(void)
{

    if (EPWM->EIPND0_b.EOP0 == 1) {
        static int16_t eopCnt = 0;
        GPIO->PDO_b.DO1       = 1;
        eopCnt++;
        if (eopCnt == MCM_CTRL_PERIOD) {
            MCM_StateMeachine();
            eopCnt = 0;
        }
        EPWM->EIPND0    = CLEAR_EPWM_ENDPERIOD_INT;
        GPIO->PDO_b.DO1 = 0;
    }
}
