/************************************************************
 * @file: MOT.c
 * @author: Young Leon
 * @version: V1.0
 * @data: 2022/10/18
 * @brief: Motor c file. Implement motor controller, including
 *          starting rotation, step commutation,
 *          clockwise/counter-clock wise commutation, brake,
 *          setting EPWM duty.
 * @note:
 * @Copyright (C) 2022 Novosense All rights reserved.
 ************************************************************/
/************************************************************
 ************************************************************
 * @par Edition History
 * -V1.0  2022.10.18
 *        -first version for NSUC16x0 sensor based BLDC Motor with
 *          six step commutation.
 *
 ************************************************************/

/**
 * Commutation Table:
 *                                       +-------------------------------+
 *                                       | CCW           |  CW           |
 *   +-----------------------------------|-------------------------------|
 *   | Sector | FBK_rotor_position_u8_g  | Phase+ Phase- | Phase+ Phase- |
 *   |-------------------------------------------------------------------|
 *   |     0  |        2                 |   A      B    |   B      A    |
 *   |     1  |        6                 |   A      C    |   C      A    |
 *   |     2  |        4                 |   B      C    |   C      B    |
 *   |     3  |        5                 |   B      A    |   A      B    |
 *   |     4  |        1                 |   C      A    |   A      C    |
 *   |     5  |        3                 |   C      B    |   B      C    |
 *   +-------------------------------------------------------------------+
 *
 */

#include "MOT.h"

#if (MOT_FRWHL == 0) /* 0 passive/asynchronous low-side freewheeling, slow decay (high-side PWM) */
/** EPWM control pattern for passive low-side FRWHL. Rotation = CW, Sector = 0. */
#define MOT_CW_SECTOR_0_EPWM_CONTROL_PATTERN  (0x000000C1)
/** EPWM control pattern for passive low-side FRWHL. Rotation = CW, Sector = 1. */
#define MOT_CW_SECTOR_1_EPWM_CONTROL_PATTERN  (0x00000C01)
/** EPWM control pattern for passive low-side FRWHL. Rotation = CW, Sector = 2. */
#define MOT_CW_SECTOR_2_EPWM_CONTROL_PATTERN  (0x00000C10)
/** EPWM control pattern for passive low-side FRWHL. Rotation = CW, Sector = 3. */
#define MOT_CW_SECTOR_3_EPWM_CONTROL_PATTERN  (0x0000001C)
/** EPWM control pattern for passive low-side FRWHL. Rotation = CW, Sector = 4. */
#define MOT_CW_SECTOR_4_EPWM_CONTROL_PATTERN  (0x0000010C)
/** EPWM control pattern for passive low-side FRWHL. Rotation = CW, Sector = 5. */
#define MOT_CW_SECTOR_5_EPWM_CONTROL_PATTERN  (0x000001C0)

/** EPWM control pattern for passive low-side FRWHL. Rotation = CCW, Sector = 0. */
#define MOT_CCW_SECTOR_0_EPWM_CONTROL_PATTERN (0x0000001C)
/** EPWM control pattern for passive low-side FRWHL. Rotation = CCW, Sector = 1. */
#define MOT_CCW_SECTOR_1_EPWM_CONTROL_PATTERN (0x0000010C)
/** EPWM control pattern for passive low-side FRWHL. Rotation = CCW, Sector = 2. */
#define MOT_CCW_SECTOR_2_EPWM_CONTROL_PATTERN (0x000001C0)
/** EPWM control pattern for passive low-side FRWHL. Rotation = CCW, Sector = 3. */
#define MOT_CCW_SECTOR_3_EPWM_CONTROL_PATTERN (0x000000C1)
/** EPWM control pattern for passive low-side FRWHL. Rotation = CCW, Sector = 4. */
#define MOT_CCW_SECTOR_4_EPWM_CONTROL_PATTERN (0x00000C01)
/** EPWM control pattern for passive low-side FRWHL. Rotation = CCW, Sector = 5. */
#define MOT_CCW_SECTOR_5_EPWM_CONTROL_PATTERN (0x00000C10)

#define MOT_CW_SECTOR_0_BEMFC_DTS             (0x02) /* trigger source 0 = HS0, 2 = HS1, 4 = HS2 */
#define MOT_CW_SECTOR_1_BEMFC_DTS             (0x04) /* trigger source 1 = LS0, 3 = LS1, 5 = LS2 */
#define MOT_CW_SECTOR_2_BEMFC_DTS             (0x04) /* trigger source 6 = HS0|LS0, 7 = HS1|LS1, 8 = HS2|LS2 */
#define MOT_CW_SECTOR_3_BEMFC_DTS             (0x00)
#define MOT_CW_SECTOR_4_BEMFC_DTS             (0x00)
#define MOT_CW_SECTOR_5_BEMFC_DTS             (0x02)

#define MOT_CCW_SECTOR_0_BEMFC_DTS            (0x00) /* trigger source 0 = HS0, 2 = HS1, 4 = HS2 */
#define MOT_CCW_SECTOR_1_BEMFC_DTS            (0x00) /* trigger source 1 = LS0, 3 = LS1, 5 = LS2 */
#define MOT_CCW_SECTOR_2_BEMFC_DTS            (0x02) /* trigger source 6 = HS0|LS0, 7 = HS1|LS1, 8 = HS2|LS2 */
#define MOT_CCW_SECTOR_3_BEMFC_DTS            (0x02)
#define MOT_CCW_SECTOR_4_BEMFC_DTS            (0x04)
#define MOT_CCW_SECTOR_5_BEMFC_DTS            (0x04)

#elif (MOT_FRWHL == 1) /* 1 passive/asynchronous high-side freewheeling, slow decay (low-side PWM) */
/** EPWM control pattern for passive high-side FRWHL. Rotation = CW, Sector = 0. */
#define MOT_CW_SECTOR_0_EPWM_CONTROL_PATTERN  (0x00000043)
/** EPWM control pattern for passive high-side FRWHL. Rotation = CW, Sector = 1. */
#define MOT_CW_SECTOR_1_EPWM_CONTROL_PATTERN  (0x00000403)
/** EPWM control pattern for passive high-side FRWHL. Rotation = CW, Sector = 2. */
#define MOT_CW_SECTOR_2_EPWM_CONTROL_PATTERN  (0x00000430)
/** EPWM control pattern for passive high-side FRWHL. Rotation = CW, Sector = 3. */
#define MOT_CW_SECTOR_3_EPWM_CONTROL_PATTERN  (0x00000034)
/** EPWM control pattern for passive high-side FRWHL. Rotation = CW, Sector = 4. */
#define MOT_CW_SECTOR_4_EPWM_CONTROL_PATTERN  (0x00000304)
/** EPWM control pattern for passive high-side FRWHL. Rotation = CW, Sector = 5. */
#define MOT_CW_SECTOR_5_EPWM_CONTROL_PATTERN  (0x00000340)

/** EPWM control pattern for passive high-side FRWHL. Rotation = CCW, Sector = 0. */
#define MOT_CCW_SECTOR_0_EPWM_CONTROL_PATTERN (0x00000034)
/** EPWM control pattern for passive high-side FRWHL. Rotation = CCW, Sector = 1. */
#define MOT_CCW_SECTOR_1_EPWM_CONTROL_PATTERN (0x00000304)
/** EPWM control pattern for passive high-side FRWHL. Rotation = CCW, Sector = 2. */
#define MOT_CCW_SECTOR_2_EPWM_CONTROL_PATTERN (0x00000340)
/** EPWM control pattern for passive high-side FRWHL. Rotation = CCW, Sector = 3. */
#define MOT_CCW_SECTOR_3_EPWM_CONTROL_PATTERN (0x00000043)
/** EPWM control pattern for passive high-side FRWHL. Rotation = CCW, Sector = 4. */
#define MOT_CCW_SECTOR_4_EPWM_CONTROL_PATTERN (0x00000403)
/** EPWM control pattern for passive high-side FRWHL. Rotation = CCW, Sector = 5. */
#define MOT_CCW_SECTOR_5_EPWM_CONTROL_PATTERN (0x00000430)

#define MOT_CW_SECTOR_0_BEMFC_DTS             (0x01) /* trigger source 0 = HS0, 2 = HS1, 4 = HS2 */
#define MOT_CW_SECTOR_1_BEMFC_DTS             (0x01) /* trigger source 1 = LS0, 3 = LS1, 5 = LS2 */
#define MOT_CW_SECTOR_2_BEMFC_DTS             (0x03) /* trigger source 6 = HS0|LS0, 7 = HS1|LS1, 8 = HS2|LS2 */
#define MOT_CW_SECTOR_3_BEMFC_DTS             (0x03)
#define MOT_CW_SECTOR_4_BEMFC_DTS             (0x05)
#define MOT_CW_SECTOR_5_BEMFC_DTS             (0x05)

#define MOT_CCW_SECTOR_0_BEMFC_DTS            (0x03) /* trigger source 0 = HS0, 2 = HS1, 4 = HS2 */
#define MOT_CCW_SECTOR_1_BEMFC_DTS            (0x05) /* trigger source 1 = LS0, 3 = LS1, 5 = LS2 */
#define MOT_CCW_SECTOR_2_BEMFC_DTS            (0x05) /* trigger source 6 = HS0|LS0, 7 = HS1|LS1, 8 = HS2|LS2 */
#define MOT_CCW_SECTOR_3_BEMFC_DTS            (0x01)
#define MOT_CCW_SECTOR_4_BEMFC_DTS            (0x01)
#define MOT_CCW_SECTOR_5_BEMFC_DTS            (0x03)

#elif (MOT_FRWHL == 2) /* 2 active/synchronous low-side freewheeling, slow decay (complementary PWM, active high) */
/** EPWM control pattern for active low-side FRWHL. Rotation = CW, Sector = 0. */
#define MOT_CW_SECTOR_0_EPWM_CONTROL_PATTERN  (0x000000E1)
/** EPWM control pattern for active low-side FRWHL. Rotation = CW, Sector = 1. */
#define MOT_CW_SECTOR_1_EPWM_CONTROL_PATTERN  (0x00000E01)
/** EPWM control pattern for active low-side FRWHL. Rotation = CW, Sector = 2. */
#define MOT_CW_SECTOR_2_EPWM_CONTROL_PATTERN  (0x00000E10)
/** EPWM control pattern for active low-side FRWHL. Rotation = CW, Sector = 3. */
#define MOT_CW_SECTOR_3_EPWM_CONTROL_PATTERN  (0x0000001E)
/** EPWM control pattern for active low-side FRWHL. Rotation = CW, Sector = 4. */
#define MOT_CW_SECTOR_4_EPWM_CONTROL_PATTERN  (0x0000010E)
/** EPWM control pattern for active low-side FRWHL. Rotation = CW, Sector = 5. */
#define MOT_CW_SECTOR_5_EPWM_CONTROL_PATTERN  (0x000001E0)

/** EPWM control pattern for active low-side FRWHL. Rotation = CCW, Sector = 0. */
#define MOT_CCW_SECTOR_0_EPWM_CONTROL_PATTERN (0x0000001E)
/** EPWM control pattern for active low-side FRWHL. Rotation = CCW, Sector = 1. */
#define MOT_CCW_SECTOR_1_EPWM_CONTROL_PATTERN (0x0000010E)
/** EPWM control pattern for active low-side FRWHL. Rotation = CCW, Sector = 2. */
#define MOT_CCW_SECTOR_2_EPWM_CONTROL_PATTERN (0x000001E0)
/** EPWM control pattern for active low-side FRWHL. Rotation = CCW, Sector = 3. */
#define MOT_CCW_SECTOR_3_EPWM_CONTROL_PATTERN (0x000000E1)
/** EPWM control pattern for active low-side FRWHL. Rotation = CCW, Sector = 4. */
#define MOT_CCW_SECTOR_4_EPWM_CONTROL_PATTERN (0x00000E01)
/** EPWM control pattern for active low-side FRWHL. Rotation = CCW, Sector = 5. */
#define MOT_CCW_SECTOR_5_EPWM_CONTROL_PATTERN (0x00000E10)

#define MOT_CW_SECTOR_0_BEMFC_DTS             (0x02) /* trigger source 0 = HS0, 2 = HS1, 4 = HS2 */
#define MOT_CW_SECTOR_1_BEMFC_DTS             (0x04) /* trigger source 1 = LS0, 3 = LS1, 5 = LS2 */
#define MOT_CW_SECTOR_2_BEMFC_DTS             (0x04) /* trigger source 6 = HS0|LS0, 7 = HS1|LS1, 8 = HS2|LS2 */
#define MOT_CW_SECTOR_3_BEMFC_DTS             (0x00)
#define MOT_CW_SECTOR_4_BEMFC_DTS             (0x00)
#define MOT_CW_SECTOR_5_BEMFC_DTS             (0x02)

#define MOT_CCW_SECTOR_0_BEMFC_DTS            (0x00) /* trigger source 0 = HS0, 2 = HS1, 4 = HS2 */
#define MOT_CCW_SECTOR_1_BEMFC_DTS            (0x00) /* trigger source 1 = LS0, 3 = LS1, 5 = LS2 */
#define MOT_CCW_SECTOR_2_BEMFC_DTS            (0x02) /* trigger source 6 = HS0|LS0, 7 = HS1|LS1, 8 = HS2|LS2 */
#define MOT_CCW_SECTOR_3_BEMFC_DTS            (0x02)
#define MOT_CCW_SECTOR_4_BEMFC_DTS            (0x04)
#define MOT_CCW_SECTOR_5_BEMFC_DTS            (0x04)

#elif (MOT_FRWHL == 3) /* 3 active/synchronous high-side freewheeling, slow decay (complementary PWM, active low) */
/** EPWM control pattern for active high-side FRWHL. Rotation = CW, Sector = 0. */
#define MOT_CW_SECTOR_0_EPWM_CONTROL_PATTERN  (0x0000004B)
/** EPWM control pattern for active high-side FRWHL. Rotation = CW, Sector = 1. */
#define MOT_CW_SECTOR_1_EPWM_CONTROL_PATTERN  (0x0000040B)
/** EPWM control pattern for active high-side FRWHL. Rotation = CW, Sector = 2. */
#define MOT_CW_SECTOR_2_EPWM_CONTROL_PATTERN  (0x000004B0)
/** EPWM control pattern for active high-side FRWHL. Rotation = CW, Sector = 3. */
#define MOT_CW_SECTOR_3_EPWM_CONTROL_PATTERN  (0x000000B4)
/** EPWM control pattern for active high-side FRWHL. Rotation = CW, Sector = 4. */
#define MOT_CW_SECTOR_4_EPWM_CONTROL_PATTERN  (0x00000B04)
/** EPWM control pattern for active high-side FRWHL. Rotation = CW, Sector = 5. */
#define MOT_CW_SECTOR_5_EPWM_CONTROL_PATTERN  (0x00000B40)
/** EPWM control pattern for active high-side FRWHL. Rotation = CCW, Sector = 0. */
#define MOT_CCW_SECTOR_0_EPWM_CONTROL_PATTERN (0x000000B4)
/** EPWM control pattern for active high-side FRWHL. Rotation = CCW, Sector = 1. */
#define MOT_CCW_SECTOR_1_EPWM_CONTROL_PATTERN (0x00000B04)
/** EPWM control pattern for active high-side FRWHL. Rotation = CCW, Sector = 2. */
#define MOT_CCW_SECTOR_2_EPWM_CONTROL_PATTERN (0x00000B40)
/** EPWM control pattern for active high-side FRWHL. Rotation = CCW, Sector = 3. */
#define MOT_CCW_SECTOR_3_EPWM_CONTROL_PATTERN (0x0000004B)
/** EPWM control pattern for active high-side FRWHL. Rotation = CCW, Sector = 4. */
#define MOT_CCW_SECTOR_4_EPWM_CONTROL_PATTERN (0x0000040B)
/** EPWM control pattern for active high-side FRWHL. Rotation = CCW, Sector = 5. */
#define MOT_CCW_SECTOR_5_EPWM_CONTROL_PATTERN (0x000004B0)

#define MOT_CW_SECTOR_0_BEMFC_DTS             (0x01) /* trigger source 0 = HS0, 2 = HS1, 4 = HS2 */
#define MOT_CW_SECTOR_1_BEMFC_DTS             (0x01) /* trigger source 1 = LS0, 3 = LS1, 5 = LS2 */
#define MOT_CW_SECTOR_2_BEMFC_DTS             (0x03) /* trigger source 6 = HS0|LS0, 7 = HS1|LS1, 8 = HS2|LS2 */
#define MOT_CW_SECTOR_3_BEMFC_DTS             (0x03)
#define MOT_CW_SECTOR_4_BEMFC_DTS             (0x05)
#define MOT_CW_SECTOR_5_BEMFC_DTS             (0x05)

#define MOT_CCW_SECTOR_0_BEMFC_DTS            (0x03) /* trigger source 0 = HS0, 2 = HS1, 4 = HS2 */
#define MOT_CCW_SECTOR_1_BEMFC_DTS            (0x05) /* trigger source 1 = LS0, 3 = LS1, 5 = LS2 */
#define MOT_CCW_SECTOR_2_BEMFC_DTS            (0x05) /* trigger source 6 = HS0|LS0, 7 = HS1|LS1, 8 = HS2|LS2 */
#define MOT_CCW_SECTOR_3_BEMFC_DTS            (0x01)
#define MOT_CCW_SECTOR_4_BEMFC_DTS            (0x01)
#define MOT_CCW_SECTOR_5_BEMFC_DTS            (0x03)

#elif (MOT_FRWHL ==                                                                                                    \
       4) /* 4 passive/asynchronous freewheeling with fast decay (high- and low-side PWM, supply feedback) */
/** EPWM control pattern for passive fast decay. Rotation = CW, Sector = 0. */
#define MOT_CW_SECTOR_0_EPWM_CONTROL_PATTERN  (0x000000C3)
/** EPWM control pattern for passive fast decay. Rotation = CW, Sector = 1. */
#define MOT_CW_SECTOR_1_EPWM_CONTROL_PATTERN  (0x00000C03)
/** EPWM control pattern for passive fast decay. Rotation = CW, Sector = 2. */
#define MOT_CW_SECTOR_2_EPWM_CONTROL_PATTERN  (0x00000C30)
/** EPWM control pattern for passive fast decay. Rotation = CW, Sector = 3. */
#define MOT_CW_SECTOR_3_EPWM_CONTROL_PATTERN  (0x0000003C)
/** EPWM control pattern for passive fast decay. Rotation = CW, Sector = 4. */
#define MOT_CW_SECTOR_4_EPWM_CONTROL_PATTERN  (0x0000030C)
/** EPWM control pattern for passive fast decay. Rotation = CW, Sector = 5. */
#define MOT_CW_SECTOR_5_EPWM_CONTROL_PATTERN  (0x000003C0)
/** EPWM control pattern for passive fast decay. Rotation = CCW, Sector = 0. */
#define MOT_CCW_SECTOR_0_EPWM_CONTROL_PATTERN (0x0000003C)
/** EPWM control pattern for passive fast decay. Rotation = CCW, Sector = 1. */
#define MOT_CCW_SECTOR_1_EPWM_CONTROL_PATTERN (0x0000030C)
/** EPWM control pattern for passive fast decay. Rotation = CCW, Sector = 2. */
#define MOT_CCW_SECTOR_2_EPWM_CONTROL_PATTERN (0x000003C0)
/** EPWM control pattern for passive fast decay. Rotation = CCW, Sector = 3. */
#define MOT_CCW_SECTOR_3_EPWM_CONTROL_PATTERN (0x000000C3)
/** EPWM control pattern for passive fast decay. Rotation = CCW, Sector = 4. */
#define MOT_CCW_SECTOR_4_EPWM_CONTROL_PATTERN (0x00000C03)
/** EPWM control pattern for passive fast decay. Rotation = CCW, Sector = 5. */
#define MOT_CCW_SECTOR_5_EPWM_CONTROL_PATTERN (0x00000C30)

#define MOT_CW_SECTOR_0_BEMFC_DTS             (0x02) /* trigger source 0 = HS0, 2 = HS1, 4 = HS2 */
#define MOT_CW_SECTOR_1_BEMFC_DTS             (0x04) /* trigger source 1 = LS0, 3 = LS1, 5 = LS2 */
#define MOT_CW_SECTOR_2_BEMFC_DTS             (0x04) /* trigger source 6 = HS0|LS0, 7 = HS1|LS1, 8 = HS2|LS2 */
#define MOT_CW_SECTOR_3_BEMFC_DTS             (0x00)
#define MOT_CW_SECTOR_4_BEMFC_DTS             (0x00)
#define MOT_CW_SECTOR_5_BEMFC_DTS             (0x02)

#define MOT_CCW_SECTOR_0_BEMFC_DTS            (0x00) /* trigger source 0 = HS0, 2 = HS1, 4 = HS2 */
#define MOT_CCW_SECTOR_1_BEMFC_DTS            (0x00) /* trigger source 1 = LS0, 3 = LS1, 5 = LS2 */
#define MOT_CCW_SECTOR_2_BEMFC_DTS            (0x02) /* trigger source 6 = HS0|LS0, 7 = HS1|LS1, 8 = HS2|LS2 */
#define MOT_CCW_SECTOR_3_BEMFC_DTS            (0x02)
#define MOT_CCW_SECTOR_4_BEMFC_DTS            (0x04)
#define MOT_CCW_SECTOR_5_BEMFC_DTS            (0x04)

#else
#error MOT_FRWHL must be defined with a value from 0 to 4.
#endif

/**
 * @brief MOT motor error flag
 *
 */
uint8_t g_MOTFaultStatus;

/**
 * @brief CCW sequence table
 *
 */
uint8_t g_MOTCcwSequenceTable[8];

/**
 * @brief CW sequence table
 *
 */
uint8_t g_MOTCwSequenceTable[8];

/**
 * @brief commutation mode applied to motor phase
 *
 */
int8_t g_MOTCommutationMode;

/**
 * @brief Actual rotor direction during rotation
 *
 */
int8_t g_MOTActualRotorDir;

/**
 *  @brief Rotor Position Value, indicate hall sensor value (HC, HB, HA)
          | 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 |
          |---|---|---|---|---|---|---|---|
          | 0 | 0 | 0 | 0 | 0 | HA| HB| HC|
           Valid Range: 0-7
*/
static uint8_t g_MOTLastMotorPosition;

/**
 * @brief Value applied to EPWM compare register to obtain bridge duty on the active MOSFETS
 * Valid Range: 0-MOT_EPWM_PERIOD_COMPARE
 *
 */
uint16_t g_MOTEpwmDutySet;

/************************************************************
 * @brief: Initialize motor module
 * @return <None>
 ************************************************************/
void MOT_Init(void)
{
    /* Initialize CW sequence table*/
    g_MOTCwSequenceTable[0]            = 0;
    g_MOTCwSequenceTable[MOT_SECTOR_0] = MOT_SECTOR_1;
    g_MOTCwSequenceTable[MOT_SECTOR_1] = MOT_SECTOR_2;
    g_MOTCwSequenceTable[MOT_SECTOR_2] = MOT_SECTOR_3;
    g_MOTCwSequenceTable[MOT_SECTOR_3] = MOT_SECTOR_4;
    g_MOTCwSequenceTable[MOT_SECTOR_4] = MOT_SECTOR_5;
    g_MOTCwSequenceTable[MOT_SECTOR_5] = MOT_SECTOR_0;
    g_MOTCwSequenceTable[7]            = 7;

    /* Initialize CCW sequence table*/
    g_MOTCcwSequenceTable[0]            = 0;
    g_MOTCcwSequenceTable[MOT_SECTOR_0] = MOT_SECTOR_5;
    g_MOTCcwSequenceTable[MOT_SECTOR_1] = MOT_SECTOR_0;
    g_MOTCcwSequenceTable[MOT_SECTOR_2] = MOT_SECTOR_1;
    g_MOTCcwSequenceTable[MOT_SECTOR_3] = MOT_SECTOR_2;
    g_MOTCcwSequenceTable[MOT_SECTOR_4] = MOT_SECTOR_3;
    g_MOTCcwSequenceTable[MOT_SECTOR_5] = MOT_SECTOR_4;
    g_MOTCcwSequenceTable[7]            = 7;

    /* config EPWM*/
    SYSCTRL->LKKEYR          = SYSCTRL_UNLOCK_KEY;
    SYSCTRL->AHBCGR_b.PWMCEN = 1;
    SYSCTRL->AHBCGR_b.DACCEN = 1;
    //    SYSCTRL->APBCGR_b.CAPCMPCEN = 1;

    /*enable Charge pump*/
    SYSCTRL->HCR_b.CE      = 1;
    SYSCTRL->ANACR_b.ISNGC = 3;

    EPWM->CMP0 = 0; // set compoare value to 0
    EPWM->CMP1 = 0;
    EPWM->CMP2 = 0;

    EPWM->PER0 = MOT_EPWM_PERIOD_COMPARE; // set pwm period
    EPWM->PER1 = MOT_EPWM_PERIOD_COMPARE;
    EPWM->PER2 = MOT_EPWM_PERIOD_COMPARE;

    EPWM->CR2_b.PRBE0 = 1; // enable register buffer
    EPWM->CR2_b.PRBE1 = 1;
    EPWM->CR2_b.PRBE2 = 1;
    EPWM->CR2_b.PRUM0 = 0; // PERx, COMPx, TRGx register update on
    EPWM->CR2_b.PRUM0 = 0; // 0: update request(PURx) 1: without
    EPWM->CR2_b.PRUM0 = 0; // update request (PURx)

    EPWM->CR1 = 0; // MOUT HS/LS mux to constaint low

    EPWM->CR0_b.FSON  = MOT_EPWM_FSON;  // set switch on slew rate
    EPWM->CR0_b.FSOFF = MOT_EPWM_FSOFF; // set switch off slew rate
    EPWM->CR0_b.CCPT  = 0;              // set cross-current to 0
    EPWM->CR0_b.CKSEL = MOT_EPWM_FSEL;  // select fpwm
    EPWM->CR0_b.OCFT  = MOT_EPWM_OCFT;  // select over-current filter time
    EPWM->CR0_b.BCM   = 1;              // set bridge mode: 0:paired 1:single
    EPWM->CR0_b.PAM   = MOT_EPWM_PAM;   // set alignment:0:center 1:zero

    EPWM->IEN0_b.EOP0 = 1; // enable EPWM0 End of period interrupt
    EPWM->IEN1_b.EOP1 = 1;
    LL_NVIC_SetPriority(EPWM1_IRQn, 0, MOT_EPWM1_IRQ_PRIO);
    LL_NVIC_DisableIRQ(EPWM1_IRQn);
    LL_NVIC_EnableIRQ(EPWM0_IRQn);
    LL_NVIC_SetPriority(EPWM0_IRQn, 0, MOT_EPWM0_IRQ_PRIO);
    EPWM->CR0_b.EN = 1; // enable EPWM connect to MOUT
}

/************************************************************
 * @brief: Switch all bridge off.
 * @return <None>
 ************************************************************/
void MOT_SwitchMotorOff(void)
{
    /* disable hall interrupt in case of conflictint with MOT_CommutatePhases*/
    FBK_DisableBEMFCCommutation();

    /* immediately swtich off MOUT*/
    EPWM->CR1 = 0;

    g_MOTCommutationMode = MOT_COMMUTATION_DISABLED;
    g_MOTActualRotorDir  = MOT_UNDEF_DIR;

    /* switch MCM.c state machine status */
    MCM_SetMotorOff();
}

/************************************************************
 * @brief: Commutation for clockwise rotation.
 * @return <None>
 ************************************************************/
static void MOT_CommutatePhasesCW(void)
{
    switch (g_FBKRotorPosition) {
    case MOT_SECTOR_0:
        EPWM->CR1 = MOT_CW_SECTOR_0_EPWM_CONTROL_PATTERN;
        LL_NVIC_EnableIRQ(EPWM1_IRQn);
        break;
    case MOT_SECTOR_1:
        EPWM->CR1 = MOT_CW_SECTOR_1_EPWM_CONTROL_PATTERN;
        LL_NVIC_EnableIRQ(EPWM1_IRQn);
        break;
    case MOT_SECTOR_2:
        EPWM->CR1 = MOT_CW_SECTOR_2_EPWM_CONTROL_PATTERN;
        LL_NVIC_EnableIRQ(EPWM1_IRQn);
        break;
    case MOT_SECTOR_3:
        EPWM->CR1 = MOT_CW_SECTOR_3_EPWM_CONTROL_PATTERN;
        LL_NVIC_EnableIRQ(EPWM1_IRQn);
        break;
    case MOT_SECTOR_4:
        EPWM->CR1 = MOT_CW_SECTOR_4_EPWM_CONTROL_PATTERN;
        LL_NVIC_EnableIRQ(EPWM1_IRQn);
        break;
    case MOT_SECTOR_5:
        EPWM->CR1 = MOT_CW_SECTOR_5_EPWM_CONTROL_PATTERN;
        LL_NVIC_EnableIRQ(EPWM1_IRQn);
        break;
    default:
        MOT_SwitchMotorOff(); /* switch the motor off; bridge high impedance */
        g_MOTFaultStatus = 1; /* indicate sector error */
        break;
    }
#if (FBK_CAPCOM_MODE == 1)
    CAPCOM->CCRPR_b.REFP = g_MOTCwSequenceTable[g_FBKRotorPosition];
#endif
}

/************************************************************
 * @brief: Commutation for counter-clockwise rotation.
 * @return <None>
 ************************************************************/
static void MOT_CommutatePhasesCCW(void)
{
    switch (g_FBKRotorPosition) {
    case MOT_SECTOR_0:
        EPWM->CR1 = MOT_CCW_SECTOR_0_EPWM_CONTROL_PATTERN;
        LL_NVIC_EnableIRQ(EPWM1_IRQn);
        break;
    case MOT_SECTOR_1:
        EPWM->CR1 = MOT_CCW_SECTOR_1_EPWM_CONTROL_PATTERN;
        LL_NVIC_EnableIRQ(EPWM1_IRQn);
        break;
    case MOT_SECTOR_2:
        EPWM->CR1 = MOT_CCW_SECTOR_2_EPWM_CONTROL_PATTERN;
        LL_NVIC_EnableIRQ(EPWM1_IRQn);
        break;
    case MOT_SECTOR_3:
        EPWM->CR1 = MOT_CCW_SECTOR_3_EPWM_CONTROL_PATTERN;
        LL_NVIC_EnableIRQ(EPWM1_IRQn);
        break;
    case MOT_SECTOR_4:
        EPWM->CR1 = MOT_CCW_SECTOR_4_EPWM_CONTROL_PATTERN;
        LL_NVIC_EnableIRQ(EPWM1_IRQn);
        break;
    case MOT_SECTOR_5:
        EPWM->CR1 = MOT_CCW_SECTOR_5_EPWM_CONTROL_PATTERN;
        LL_NVIC_EnableIRQ(EPWM1_IRQn);
        break;
    default:
        MOT_SwitchMotorOff(); /* switch the motor off; bridge high impedance */
        g_MOTFaultStatus = 1; /* indicate sector error */
        break;
    }
#if (FBK_CAPCOM_MODE == 1)
    CAPCOM->CCRPR_b.REFP = g_MOTCcwSequenceTable[g_FBKRotorPosition];
#endif
}

/************************************************************
 * @brief: Start counter-clockwise rotate.
 * @return <None>
 ************************************************************/
void MOT_StartCCWRotation(void)
{
    /* The Interrupts are disabled to avoid conflict with MOT_CommutatePhases */
    FBK_DisableBEMFCCommutation();

    /* Set CCW rotation initial poition */
    g_FBKRotorPosition     = g_MOTCcwSequenceTable[g_FBKRotorPosition];
    g_MOTLastMotorPosition = g_FBKRotorPosition;

    /* Set PWM width to be used during phases commutation */
    g_MOTEpwmDutySet = MOT_SetCompareValue(g_MCMPwmWidth);

    EPWM->CMP0 = g_MOTEpwmDutySet;
    EPWM->CMP1 = g_MOTEpwmDutySet;
    EPWM->CMP2 = g_MOTEpwmDutySet;

    EPWM->CR2 |= 0x00000700; /* set update request flags PUR0, PUR1 and PUR2 */

    g_MOTCommutationMode = MOT_CCW_COMMUTATION_ENABLED;

    /* The Interrupts now can be enabled */
    FBK_EnableBEMFCCommutation();
}

/************************************************************
 * @brief: Start clockwise rotate.
 * @return <None>
 ************************************************************/
void MOT_StartCWRotation(void)
{
    /* The Interrupts are disabled to avoid conflict with MOT_CommutatePhases */
    FBK_DisableBEMFCCommutation();

    /* Set CW rotation initial poition */
    g_FBKRotorPosition     = g_MOTCwSequenceTable[g_FBKRotorPosition];
    g_MOTLastMotorPosition = g_FBKRotorPosition;

    /* Set the compare value to obtain the required PWM width (g_MCMPwmWidth) */
    g_MOTEpwmDutySet = MOT_SetCompareValue(g_MCMPwmWidth);

    EPWM->CMP0 = g_MOTEpwmDutySet;
    EPWM->CMP1 = g_MOTEpwmDutySet;
    EPWM->CMP2 = g_MOTEpwmDutySet;

    EPWM->CR2 |= 0x00000700; /* set update request flags PUR0, PUR1 and PUR2 */

    g_MOTCommutationMode = MOT_CW_COMMUTATION_ENABLED;

    /* The Interrupts now can be enabled */
    FBK_EnableBEMFCCommutation();
}

/************************************************************
 * @brief: Load EPWM duty into EPWM in clockwise rotation
 * @return <None>
 ************************************************************/
void MOT_ChangeEpwmWidth(void)
{
    /* Set the compare value to obtain the required PWM width (g_MCMPwmWidth)
     */
    g_MOTEpwmDutySet = MOT_SetCompareValue(g_MCMPwmWidth);

    /* in case a reload is pending, wait until reload is finished, wait <= one EPWM
     * period */
    while ((EPWM->CR2 & 0x00000700) != 0) {
    };

    EPWM->CMP0 = g_MOTEpwmDutySet;
    EPWM->CMP1 = g_MOTEpwmDutySet;
    EPWM->CMP2 = g_MOTEpwmDutySet;

    EPWM->CR2 |= 0x00000700; /* set update request flags PUR0, PUR1 and PUR2 */
}

/************************************************************
 * @brief: Brake motor by switch off all high bridge and all low bridge on.
 * @return <None>
 ************************************************************/
void MOT_BrakeMotor(void)
{
    /* The Hall Interrupts are disabled to avoid conflict with MOT_CommutatePhases */
    FBK_DisableBEMFCCommutation();

    /* immediate break; MOUTx high side off and low side on */
    EPWM->CR1 = 0x00111111;

    g_MOTEpwmDutySet = MOT_COMPARE_VALUE_OFF;

    /* Disable the motor phases commutation */
    g_MOTCommutationMode = MOT_COMMUTATION_DISABLED;
}

/************************************************************
 * @brief: Commutate motor phase using CAPCOM 0,1,2 input
 * @return <None>
 ************************************************************/
void MOT_CommutatePhases(void)
{
    /* Latch the rotor position value for next check */
    g_MOTLastMotorPosition = g_FBKRotorPosition;
    /* Run the phases commutation accoding to the enabled direction */
    if (g_MOTCommutationMode == MOT_CCW_COMMUTATION_ENABLED) {
        g_FBKRotorPosition = g_MOTCcwSequenceTable[g_FBKRotorPosition];
        MOT_CommutatePhasesCCW();

    } else if (g_MOTCommutationMode == MOT_CW_COMMUTATION_ENABLED) {
        g_FBKRotorPosition = g_MOTCwSequenceTable[g_FBKRotorPosition];
        MOT_CommutatePhasesCW();
    } else { /* KO, Rotor Position/Sequence is NOT correct */
        /* Switch the motor OFF to avoid WRONG phases commutation*/
        MOT_SwitchMotorOff();
        g_MOTFaultStatus = 1;
    }
}

/************************************************************
 * @brief: Fetch actual rotor direction.
 * @return <None>
 ************************************************************/
void MOT_UpdateActualRotorDirection(void)
{
    /* Is it valid? Is it in the correct sequence with the previous rotor position?
     */
    if (g_FBKRotorPosition == g_MOTCcwSequenceTable[g_MOTLastMotorPosition]) { /* YES, the rotor position is correctly
                                                                     going to CCW direction */
        g_MOTActualRotorDir = MOT_CCW;
    } else if (g_FBKRotorPosition == g_MOTCwSequenceTable[g_MOTLastMotorPosition]) { /* YES, the rotor position is
                                                                            correctly going to CW direction
                                                                          */
        g_MOTActualRotorDir = MOT_CW;
    } else { /* ERROR, the actual rotor position is not following a right sequence */
        g_MOTActualRotorDir = MOT_UNDEF_DIR;
    }
}

/************************************************************
 * @brief: Start clockwise step commutation
 * @return <None>
 ************************************************************/
void MOT_StartCWStep(void)
{
    /* Disable the motor phases commutation */
    g_MOTCommutationMode = MOT_COMMUTATION_DISABLED;

    /* Set the compare value to obtain the required PWM width (g_MCMPwmWidth)
     */
    g_MOTEpwmDutySet = MOT_SetCompareValue(g_MCMPwmWidth);

    /* in case a reload is pending, wait until reload is finished, wait <= one EPWM
     * period */
    while ((EPWM->CR2 & 0x00000700) != 0) {
    };

    EPWM->CMP0 = g_MOTEpwmDutySet;
    EPWM->CMP1 = g_MOTEpwmDutySet;
    EPWM->CMP2 = g_MOTEpwmDutySet;
    EPWM->CR2 |= 0x00000700; /* set update request flags PUR0, PUR1 and PUR2 */

    MOT_CommutatePhasesCW();
}

/************************************************************
 * @brief: Start counter-clockwise step commutation
 * @return <None>
 ************************************************************/
void MOT_StartCCWStep(void)
{
    /* Disable the motor phases commutation */
    g_MOTCommutationMode = MOT_COMMUTATION_DISABLED;

    /* Set the compare value to obtain the required PWM width (g_MCMPwmWidth)
     */
    g_MOTEpwmDutySet = MOT_SetCompareValue(g_MCMPwmWidth);

    /* in case a reload is pending, wait until reload is finished, wait <= one EPWM
     * period */
    while ((EPWM->CR2 & 0x00000700) != 0) {
    };

    EPWM->CMP0 = g_MOTEpwmDutySet;
    EPWM->CMP1 = g_MOTEpwmDutySet;
    EPWM->CMP2 = g_MOTEpwmDutySet;
    EPWM->CR2 |= 0x00000700; /* set update request flags PUR0, PUR1 and PUR2 */

    MOT_CommutatePhasesCCW();
}

/************************************************************
 * @brief: One step commutation for clockwise rotation
 * @return <None>
 ************************************************************/
void MOT_GoOneStepCW(void)
{
    g_FBKRotorPosition = g_MOTCwSequenceTable[g_FBKRotorPosition];
    /* Set the compare value to obtain the required PWM width (g_MCMPwmWidth)
     */
    g_MOTEpwmDutySet = MOT_SetCompareValue(g_MCMPwmWidth);
    MOT_ChangeEpwmWidth();
    MOT_CommutatePhasesCW();
}

/************************************************************
 * @brief: One step commutation for counter-clockwise rotation
 * @return <None>
 ************************************************************/
void MOT_GoOneStepCCW(void)
{
    g_FBKRotorPosition = g_MOTCcwSequenceTable[g_FBKRotorPosition];
    /* Set the compare value to obtain the required PWM width (g_MCMPwmWidth)
     */
    g_MOTEpwmDutySet = MOT_SetCompareValue(g_MCMPwmWidth);
    MOT_ChangeEpwmWidth();
    MOT_CommutatePhasesCCW();
}

/************************************************************
 * @brief: Leave step commutation mode
 * @return <None>
 ************************************************************/
void MOT_LeaveMotorStep(void)
{
    /* The motor is switched off before any transition */
    MOT_SwitchMotorOff();
}

/************************************************************
 * @brief: EPWM1 interrupt handler.
 * @return <None>
 ************************************************************/
void EPWM1_IRQHandler(void)
{
    static uint16_t eopCounter = 0;

    if (EPWM->EIPND1_b.EOP1) {
        eopCounter++;
        EPWM->EIPND1_b.EOP1 = 1;
    }

    /* process demagnetization and set proper capture type*/
    if (eopCounter == MOT_DEMAGNETIZATION_TIME) {
        eopCounter = 0;
        LL_NVIC_DisableIRQ(EPWM1_IRQn);
        switch (g_FBKRotorPosition) {
        case MOT_SECTOR_0:
            if (g_MOTCommutationMode == MOT_CW_COMMUTATION_ENABLED) {
                BEMFC->CR1_b.DTS1 = MOT_CW_SECTOR_0_BEMFC_DTS;
            } else {
                BEMFC->CR1_b.DTS1 = MOT_CCW_SECTOR_0_BEMFC_DTS;
            }
            CAPCOM->CCR_b.IAM1 = 0x2;
            CAPCOM->IEN_b.CAP1 = 1;
            break;
        case MOT_SECTOR_1:
            if (g_MOTCommutationMode == MOT_CW_COMMUTATION_ENABLED) {
                BEMFC->CR1_b.DTS2 = MOT_CW_SECTOR_1_BEMFC_DTS;
            } else {
                BEMFC->CR1_b.DTS2 = MOT_CCW_SECTOR_1_BEMFC_DTS;
            }
            CAPCOM->CCR_b.IAM2 = 0x1;
            CAPCOM->IEN_b.CAP2 = 1;
            break;
        case MOT_SECTOR_2:
            if (g_MOTCommutationMode == MOT_CW_COMMUTATION_ENABLED) {
                BEMFC->CR1_b.DTS0 = MOT_CW_SECTOR_2_BEMFC_DTS;
            } else {
                BEMFC->CR1_b.DTS0 = MOT_CCW_SECTOR_2_BEMFC_DTS;
            }
            CAPCOM->CCR_b.IAM0 = 0x2;
            CAPCOM->IEN_b.CAP0 = 1;
            break;
        case MOT_SECTOR_3:
            if (g_MOTCommutationMode == MOT_CW_COMMUTATION_ENABLED) {
                BEMFC->CR1_b.DTS1 = MOT_CW_SECTOR_3_BEMFC_DTS;
            } else {
                BEMFC->CR1_b.DTS1 = MOT_CCW_SECTOR_3_BEMFC_DTS;
            }
            CAPCOM->CCR_b.IAM1 = 0x1;
            CAPCOM->IEN_b.CAP1 = 1;
            break;
        case MOT_SECTOR_4:
            if (g_MOTCommutationMode == MOT_CW_COMMUTATION_ENABLED) {
                BEMFC->CR1_b.DTS2 = MOT_CW_SECTOR_4_BEMFC_DTS;
            } else {
                BEMFC->CR1_b.DTS2 = MOT_CCW_SECTOR_4_BEMFC_DTS;
            }
            CAPCOM->CCR_b.IAM2 = 0x2;
            CAPCOM->IEN_b.CAP2 = 1;
            break;
        case MOT_SECTOR_5:
            if (g_MOTCommutationMode == MOT_CW_COMMUTATION_ENABLED) {
                BEMFC->CR1_b.DTS0 = MOT_CW_SECTOR_5_BEMFC_DTS;
            } else {
                BEMFC->CR1_b.DTS0 = MOT_CCW_SECTOR_5_BEMFC_DTS;
            }
            CAPCOM->CCR_b.IAM0 = 0x1;
            CAPCOM->IEN_b.CAP0 = 1;
            break;
        default:
            MOT_SwitchMotorOff();
            g_MOTFaultStatus = 1;
            break;
        }
    }
}
